#include "wtypes.hh"
#include "wframecache.hh"
#include "ParseLine.hh"
#include "matlab_fcs.hh"
#include <sstream>
#include <iostream>

using namespace wpipe;
using namespace std;

//=====================================  frame cache constructor.
wframecache::wframecache(const std::string& file) {
  ParseLine pl(file.c_str());
  if (!pl.isOpen()) {
    cerr << "Unable to open frame cache file: " << file << endl;
    error("Can not open frame cache file");
  }
  while (pl.getLine() >= 0) {
    int nArg = pl.getCount();
    if (!nArg) continue;
    if (nArg < 6) {
      cerr << "Invalid format line in: " << file << endl;
      break;
    }
    string prefix = pl[0];
    prefix += string("-") + pl[1];
    gps_type start = pl.getInt(2);
    gps_type stop  = pl.getInt(3);
    gps_type delta = pl.getInt(4);
    string dir = pl[5];
    cout << "frame group prefix: " << prefix << " start: " << start
	 << " stop: " << stop << " delta: " << delta << " dir: " << dir
	 <<endl;
    frameCache.push_back(frame_group(prefix, start, stop, delta, dir));
  }
}

//=====================================  get a list of frames with specified 
//                                       data
int
wframecache::get_list(const std::string& prefix, gps_type start, 
		      gps_type stop, str_vect& frames) const {
  frames.clear();
  size_t N = frameCache.size();
  for (size_t i=0; i<N ; i++) {
    if (frameCache[i].valid(prefix, start, stop)) {
      frameCache[i].get_frames(start, stop, frames);
    }
  }
  return frames.size();
}


//=====================================  construct a frame group
wframecache::frame_group::frame_group(const std::string& prefix,
				      gps_type start, 
				      gps_type stop,
				      gps_type frame_len, 
				      const std::string& dir) 
  : _prefix(prefix), _directory(dir), _start(start), _length(frame_len),
    _end(stop)
{}

//=====================================  Test if requested data in group
bool 
wframecache::frame_group::valid(const string& prefix, gps_type start, 
				gps_type stop) const {

  bool rc = (prefix == _prefix && start < _end && stop > _start);
  return rc;
}

//=====================================  Get a list of frames
int
wframecache::frame_group::get_frames(gps_type start, gps_type stop, 
				     str_vect& frames) const {
  int count = 0;

  //-----------------------------------  Handle special entries.
  if (_length == 0) {
    frames.push_back(_directory);
    count++;
  } 

  //-----------------------------------  Build frame list.
  else {
    for (gps_type gps=_start; gps<_end; gps += _length) {
      if (gps < stop && gps + _length > start) {
	frames.push_back(frame(gps));
	count++;
      }
    }
  }
  return count;
}

//======================================  Get name os frame
std::string 
wframecache::frame_group::frame(gps_type gps) const {
  ostringstream ostr;
  ostr << _directory << "/" << _prefix << "-" << gps << "-" 
       << _length << ".gwf";
  return ostr.str();
}
